@@ -1,5 +1,5 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 
 from trytond.pool import Pool
 from .fiscalyear import *
@@ -8,6 +8,7 @@ from .configuration import *
 from .period import *
 from .journal import *
 from .move import *
+from .move_template import *
 from .tax import *
 from .party import *
 
@@ -47,6 +48,7 @@ def register():
         OpenJournalAsk,
         ReconcileLinesWriteOff,
         ReconcileShow,
+        CancelMovesDefault,
         FiscalYearLine,
         FiscalYear2,
         PrintGeneralJournalStart,
@@ -65,6 +67,12 @@ def register():
         AccountTemplate2,
         AccountTax,
         Account2,
+        MoveTemplate,
+        MoveTemplateKeyword,
+        MoveLineTemplate,
+        TaxLineTemplate,
+        CreateMoveTemplate,
+        CreateMoveKeywords,
         Party,
         module='account', type_='model')
     Pool.register(
@@ -91,6 +99,7 @@ def register():
         Reconcile,
         CancelMoves,
         PrintGeneralJournal,
+        CreateMove,
         OpenChartTaxCode,
         OpenTaxCode,
         module='account', type_='wizard')
