@@ -1,15 +1,12 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 from decimal import Decimal
 import datetime
 from itertools import groupby, combinations
 from operator import itemgetter
 from collections import defaultdict
 
-try:
-    from sql import Null
-except ImportError:
-    Null = None
+from sql import Null
 from sql.aggregate import Sum
 from sql.conditionals import Coalesce, Case
 
@@ -18,7 +15,7 @@ from trytond.wizard import Wizard, StateTransition, StateView, StateAction, \
     Button
 from trytond.report import Report
 from trytond import backend
-from trytond.pyson import Eval, Bool, PYSONEncoder
+from trytond.pyson import Eval, Bool, PYSONEncoder, If
 from trytond.transaction import Transaction
 from trytond.pool import Pool, PoolMeta
 from trytond.rpc import RPC
@@ -30,7 +27,7 @@ __all__ = ['Move', 'Reconciliation', 'Line', 'OpenJournalAsk',
     'ReconcileLinesWriteOff', 'ReconcileLines',
     'UnreconcileLines',
     'Reconcile', 'ReconcileShow',
-    'CancelMoves',
+    'CancelMoves', 'CancelMovesDefault',
     'FiscalYearLine', 'FiscalYear2',
     'PrintGeneralJournalStart', 'PrintGeneralJournal', 'GeneralJournal']
 __metaclass__ = PoolMeta
@@ -52,8 +49,13 @@ class Move(ModelSQL, ModelView):
     number = fields.Char('Number', required=True, readonly=True)
     post_number = fields.Char('Post Number', readonly=True,
         help='Also known as Folio Number')
+    company = fields.Many2One('company.company', 'Company', required=True,
+        states=_MOVE_STATES, depends=_MOVE_DEPENDS)
     period = fields.Many2One('account.period', 'Period', required=True,
-            states=_MOVE_STATES, depends=_MOVE_DEPENDS, select=True)
+        domain=[
+            ('company', '=', Eval('company', -1)),
+            ],
+        states=_MOVE_STATES, depends=_MOVE_DEPENDS + ['company'], select=True)
     journal = fields.Many2One('account.journal', 'Journal', required=True,
             states=_MOVE_STATES, depends=_MOVE_DEPENDS)
     date = fields.Date('Effective Date', required=True, states=_MOVE_STATES,
@@ -68,7 +70,10 @@ class Move(ModelSQL, ModelView):
         ('posted', 'Posted'),
         ], 'State', required=True, readonly=True, select=True)
     lines = fields.One2Many('account.move.line', 'move', 'Lines',
-            states=_MOVE_STATES, depends=_MOVE_DEPENDS,
+        domain=[
+            ('account.company', '=', Eval('company', -1)),
+            ],
+        states=_MOVE_STATES, depends=_MOVE_DEPENDS + ['company'],
             context={
                 'journal': Eval('journal'),
                 'period': Eval('period'),
@@ -90,8 +95,6 @@ class Move(ModelSQL, ModelView):
                     '"%(move)s" to draft in journal "%(journal)s".'),
                 'modify_posted_move': ('You can not modify move "%s" because '
                     'it is already posted.'),
-                'company_in_move': ('You can not create lines on accounts'
-                    'of different companies in move "%s".'),
                 'date_outside_period': ('You can not create move "%(move)s" '
                     'because it\'s date is outside its period.'),
                 'draft_closed_period': ('You can not set to draft move '
@@ -114,6 +117,12 @@ class Move(ModelSQL, ModelView):
         TableHandler = backend.get('TableHandler')
         cursor = Transaction().cursor
         table = TableHandler(cursor, cls, module_name)
+        sql_table = cls.__table__()
+        pool = Pool()
+        Period = pool.get('account.period')
+        period = Period.__table__()
+        FiscalYear = pool.get('account.fiscalyear')
+        fiscalyear = FiscalYear.__table__()
 
         # Migration from 2.4:
         #   - name renamed into number
@@ -123,14 +132,29 @@ class Move(ModelSQL, ModelView):
         if table.column_exist('reference'):
             table.column_rename('reference', 'post_number')
 
+        created_company = not table.column_exist('company')
+
         super(Move, cls).__register__(module_name)
 
+        # Migration from 3.4: new company field
+        if created_company:
+            # Don't use UPDATE FROM because SQLite nor MySQL support it.
+            value = period.join(fiscalyear,
+                condition=period.fiscalyear == fiscalyear.id).select(
+                    fiscalyear.company,
+                    where=period.id == sql_table.period)
+            cursor.execute(*sql_table.update([sql_table.company], [value]))
+
         table = TableHandler(cursor, cls, module_name)
         table.index_action(['journal', 'period'], 'add')
 
         # Add index on create_date
         table.index_action('create_date', action='add')
 
+    @staticmethod
+    def default_company():
+        return Transaction().context.get('company')
+
     @staticmethod
     def default_period():
         Period = Pool().get('account.period')
@@ -156,6 +180,11 @@ class Move(ModelSQL, ModelView):
     def on_change_with_date(self):
         Line = Pool().get('account.move.line')
         date = self.date
+        if date:
+            if self.period and not (
+                    self.period.start_date <= date <= self.period.end_date):
+                date = self.period.start_date
+            return date
         lines = Line.search([
                 ('journal', '=', self.journal),
                 ('period', '=', self.period),
@@ -184,17 +213,8 @@ class Move(ModelSQL, ModelView):
     def validate(cls, moves):
         super(Move, cls).validate(moves)
         for move in moves:
-            move.check_company()
             move.check_date()
 
-    def check_company(self):
-        company_id = -1
-        for line in self.lines:
-            if company_id < 0:
-                company_id = line.account.company.id
-            if line.account.company.id != company_id:
-                self.raise_user_error('company_in_move', (self.rec_name,))
-
     def check_date(self):
         if (self.date < self.period.start_date
                 or self.date > self.period.end_date):
@@ -211,7 +231,11 @@ class Move(ModelSQL, ModelView):
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        return ['OR',
+        if clause[1].startswith('!') or clause[1].startswith('not '):
+            bool_op = 'AND'
+        else:
+            bool_op = 'OR'
+        return [bool_op,
             ('post_number',) + tuple(clause[1:]),
             (cls._rec_name,) + tuple(clause[1:]),
             ]
@@ -288,17 +312,10 @@ class Move(ModelSQL, ModelView):
         '''
         pool = Pool()
         MoveLine = pool.get('account.move.line')
-        User = pool.get('res.user')
         line = MoveLine.__table__()
 
         cursor = Transaction().cursor
 
-        if (Transaction().user == 0
-                and Transaction().context.get('user')):
-            user = Transaction().context.get('user')
-        else:
-            user = Transaction().user
-        company = User(user).company
         amounts = {}
         move2draft_lines = {}
         for sub_move_ids in grouped_slice([m.id for m in moves]):
@@ -313,7 +330,7 @@ class Move(ModelSQL, ModelView):
             cursor.execute(*line.select(line.move, line.id,
                     where=red_sql & (line.state == 'draft'),
                     order_by=line.move))
-            move2draft_lines.update(dict((k, (j[1] for j in g))
+            move2draft_lines.update(dict((k, [j[1] for j in g])
                     for k, g in groupby(cursor.fetchall(), itemgetter(0))))
 
         valid_moves = []
@@ -325,9 +342,8 @@ class Move(ModelSQL, ModelView):
             # SQLite uses float for SUM
             if not isinstance(amount, Decimal):
                 amount = Decimal(amount)
-            draft_lines = MoveLine.browse(
-                list(move2draft_lines.get(move.id, [])))
-            if not company.currency.is_zero(amount):
+            draft_lines = MoveLine.browse(move2draft_lines.get(move.id, []))
+            if not move.company.currency.is_zero(amount):
                 draft_moves.append(move.id)
                 continue
             if not draft_lines:
@@ -366,28 +382,20 @@ class Move(ModelSQL, ModelView):
                     })
         return default
 
-    def cancel(self):
+    def cancel(self, default=None):
         'Return a cancel move'
-        pool = Pool()
-        Line = pool.get('account.move.line')
-        TaxLine = pool.get('account.tax.line')
-        default = self._cancel_default()
+        if default is None:
+            default = {}
+        default.update(self._cancel_default())
         cancel_move, = self.copy([self], default=default)
-        lines = []
-        tax_lines = []
         for line in cancel_move.lines:
             line.debit *= -1
             line.credit *= -1
-            lines.extend(([line], line._save_values))
-            line._values = None
             for tax_line in line.tax_lines:
                 tax_line.amount *= -1
-                tax_lines.extend(([tax_line], tax_line._save_values))
-                tax_line._values = None
-        if lines:
-            Line.write(*lines)
-        if tax_lines:
-            TaxLine.write(*tax_lines)
+            line.tax_lines = line.tax_lines  # Force tax_lines changing
+        cancel_move.lines = cancel_move.lines  # Force lines changing
+        cancel_move.save()
         return cancel_move
 
     @classmethod
@@ -469,7 +477,7 @@ class Reconciliation(ModelSQL, ModelView):
                     'configured as not reconcilable.'),
                 'reconciliation_different_parties': ('You can not reconcile '
                     'line "%(line)s" because it\'s party "%(party1)s" is '
-                    'different from %(party2)s".'),
+                    'different from "%(party2)s".'),
                 'reconciliation_unbalanced': ('You can not create a '
                     'reconciliation where debit "%(debit)s" and credit '
                     '"%(credit)s" differ.'),
@@ -581,8 +589,8 @@ class Line(ModelSQL, ModelView):
         'get_move_field', setter='set_move_field',
         searcher='search_move_field')
     amount_second_currency = fields.Numeric('Amount Second Currency',
-            digits=(16, Eval('second_currency_digits', 2)),
-            help='The amount expressed in a second currency',
+        digits=(16, Eval('second_currency_digits', 2)),
+        help='The amount expressed in a second currency',
         states={
             'required': Bool(Eval('second_currency')),
             },
@@ -598,8 +606,8 @@ class Line(ModelSQL, ModelView):
             'required': Eval('party_required', False),
             'invisible': ~Eval('party_required', False),
             },
-        depends=['party_required'])
-    party_required= fields.Function(fields.Boolean('Party Required'),
+        depends=['party_required'], ondelete='RESTRICT')
+    party_required = fields.Function(fields.Boolean('Party Required'),
         'on_change_with_party_required')
     maturity_date = fields.Date('Maturity Date',
         help='This field is used for payable and receivable lines. \n'
@@ -619,6 +627,14 @@ class Line(ModelSQL, ModelView):
             'get_currency_digits')
     second_currency_digits = fields.Function(fields.Integer(
         'Second Currency Digits'), 'get_currency_digits')
+    amount = fields.Function(fields.Numeric('Amount',
+            digits=(16, Eval('amount_currency_digits', 2)),
+            depends=['amount_currency_digits']),
+        'get_amount')
+    amount_currency = fields.Function(fields.Many2One('currency.currency',
+            'Amount Currency'), 'get_amount_currency')
+    amount_currency_digits = fields.Function(fields.Integer(
+            'Amount Currency Digits'), 'get_amount_currency')
 
     @classmethod
     def __setup__(cls):
@@ -631,6 +647,10 @@ class Line(ModelSQL, ModelView):
             ('credit_debit',
                 'CHECK(credit * debit = 0.0)',
                 'Wrong credit/debit values.'),
+            ('second_currency_sign',
+                'CHECK(COALESCE(amount_second_currency, 0) '
+                '* (debit - credit) >= 0)',
+                'wrong_second_currency_sign'),
             ]
         cls.__rpc__.update({
                 'on_write': RPC(instantiate=0),
@@ -653,6 +673,7 @@ class Line(ModelSQL, ModelView):
                 'already_reconciled': 'Line "%s" (%d) already reconciled.',
                 'party_required': 'Party is required on line "%s"',
                 'party_set': 'Party must not be set on line "%s"',
+                'wrong_second_currency_sign': 'Wrong second currency sign.',
                 })
 
     @classmethod
@@ -731,7 +752,7 @@ class Line(ModelSQL, ModelView):
                 with_rec_name=with_rec_name)
 
         if 'move' not in fields:
-            #Not manual entry
+            # Not manual entry
             if 'date' in values:
                 values = values.copy()
                 del values['date']
@@ -911,54 +932,56 @@ class Line(ModelSQL, ModelView):
         return Move.get_origin()
 
     @fields.depends('account', 'debit', 'credit', 'tax_lines', 'journal',
-        'move')
+        'move', 'amount_second_currency')
     def on_change_debit(self):
-        changes = {}
         Journal = Pool().get('account.journal')
         if self.journal or Transaction().context.get('journal'):
             journal = self.journal or Journal(Transaction().context['journal'])
             if journal.type in ('expense', 'revenue'):
-                changes['tax_lines'] = self._compute_tax_lines(journal.type)
-                if not changes['tax_lines']:
-                    del changes['tax_lines']
+                self._compute_tax_lines(journal.type)
         if self.debit:
-            changes['credit'] = Decimal('0.0')
-        return changes
+            self.credit = Decimal('0.0')
+        self._amount_second_currency_sign()
 
     @fields.depends('account', 'debit', 'credit', 'tax_lines', 'journal',
-        'move')
+        'move', 'amount_second_currency')
     def on_change_credit(self):
-        changes = {}
         Journal = Pool().get('account.journal')
         if self.journal or Transaction().context.get('journal'):
             journal = self.journal or Journal(Transaction().context['journal'])
             if journal.type in ('expense', 'revenue'):
-                changes['tax_lines'] = self._compute_tax_lines(journal.type)
-                if not changes['tax_lines']:
-                    del changes['tax_lines']
+                self._compute_tax_lines(journal.type)
         if self.credit:
-            changes['debit'] = Decimal('0.0')
-        return changes
+            self.debit = Decimal('0.0')
+        self._amount_second_currency_sign()
+
+    @fields.depends('amount_second_currency', 'debit', 'credit')
+    def on_change_amount_second_currency(self):
+        self._amount_second_currency_sign()
+
+    def _amount_second_currency_sign(self):
+        'Set correct sign to amount_second_currency'
+        if self.amount_second_currency:
+            self.amount_second_currency = \
+                self.amount_second_currency.copy_sign(self.debit - self.credit)
 
     @fields.depends('account', 'debit', 'credit', 'tax_lines', 'journal',
         'move')
     def on_change_account(self):
         Journal = Pool().get('account.journal')
 
-        changes = {}
         if Transaction().context.get('journal'):
             journal = Journal(Transaction().context['journal'])
             if journal.type in ('expense', 'revenue'):
-                changes['tax_lines'] = self._compute_tax_lines(journal.type)
-                if not changes['tax_lines']:
-                    del changes['tax_lines']
+                self._compute_tax_lines(journal.type)
 
         if self.account:
-            changes['currency_digits'] = self.account.currency_digits
+            self.currency_digits = self.account.currency_digits
             if self.account.second_currency:
-                changes['second_currency_digits'] = \
+                self.second_currency_digits = \
                     self.account.second_currency.digits
-        return changes
+            if not self.account.party_required:
+                self.party = None
 
     @fields.depends('account')
     def on_change_with_party_required(self, name=None):
@@ -967,17 +990,14 @@ class Line(ModelSQL, ModelView):
         return False
 
     def _compute_tax_lines(self, journal_type):
-        res = {}
         pool = Pool()
-        TaxCode = pool.get('account.tax.code')
         Tax = pool.get('account.tax')
         TaxLine = pool.get('account.tax.line')
 
         if self.move:
-            #Only for first line
-            return res
-        if self.tax_lines:
-            res['remove'] = [x['id'] for x in self.tax_lines]
+            # Only for first line
+            return
+        tax_lines = []
         if self.account:
             debit = self.debit or Decimal('0.0')
             credit = self.credit or Decimal('0.0')
@@ -1006,36 +1026,33 @@ class Line(ModelSQL, ModelView):
                 for code_id, tax_id in base_amounts:
                     if not base_amounts[code_id, tax_id]:
                         continue
-                    value = TaxLine.default_get(TaxLine._fields.keys())
-                    value.update({
-                            'amount': base_amounts[code_id, tax_id],
-                            'currency_digits': self.account.currency_digits,
-                            'code': code_id,
-                            'code.rec_name': TaxCode(code_id).rec_name,
-                            'tax': tax_id,
-                            'tax.rec_name': Tax(tax_id).rec_name,
-                            })
-                    res.setdefault('add', []).append(value)
-        return res
+                    tax_line = TaxLine(**TaxLine.default_get(
+                            TaxLine._fields.keys()))
+
+                    tax_line.amount = base_amounts[code_id, tax_id],
+                    tax_line.currency_digits = self.account.currency_digits
+                    tax_line.code = code_id
+                    tax_line.tax = tax_id
+                    tax_lines.append(tax_line)
+        self.tax_lines = tax_lines
 
     @fields.depends('move', 'party', 'account', 'debit', 'credit', 'journal')
     def on_change_party(self):
         Journal = Pool().get('account.journal')
         cursor = Transaction().cursor
-        res = {}
         if (not self.party) or self.account:
-            return res
+            return
 
         if not self.party.account_receivable \
                 or not self.party.account_payable:
-            return res
+            return
 
         if self.party and (not self.debit) and (not self.credit):
             type_name = self.__class__.debit.sql_type().base
             table = self.__table__()
             column = Coalesce(Sum(Coalesce(table.debit, 0)
                     - Coalesce(table.credit, 0)), 0).cast(type_name)
-            where = ((table.reconciliation == None)
+            where = ((table.reconciliation == Null)
                 & (table.party == self.party.id))
             cursor.execute(*table.select(column,
                     where=where
@@ -1046,16 +1063,14 @@ class Line(ModelSQL, ModelView):
                 amount = Decimal(str(amount))
             if not self.party.account_receivable.currency.is_zero(amount):
                 if amount > Decimal('0.0'):
-                    res['credit'] = \
+                    self.credit = \
                         self.party.account_receivable.currency.round(amount)
-                    res['debit'] = Decimal('0.0')
+                    self.debit = Decimal('0.0')
                 else:
-                    res['credit'] = Decimal('0.0')
-                    res['debit'] = \
+                    self.credit = Decimal('0.0')
+                    self.debit = \
                         - self.party.account_receivable.currency.round(amount)
-                res['account'] = self.party.account_receivable.id
-                res['account.rec_name'] = \
-                    self.party.account_receivable.rec_name
+                self.account = self.party.account_receivable
             else:
                 cursor.execute(*table.select(column,
                         where=where
@@ -1063,40 +1078,30 @@ class Line(ModelSQL, ModelView):
                 amount = cursor.fetchone()[0]
                 if not self.party.account_payable.currency.is_zero(amount):
                     if amount > Decimal('0.0'):
-                        res['credit'] = \
+                        self.credit = \
                             self.party.account_payable.currency.round(amount)
-                        res['debit'] = Decimal('0.0')
+                        self.debit = Decimal('0.0')
                     else:
-                        res['credit'] = Decimal('0.0')
-                        res['debit'] = \
+                        self.credit = Decimal('0.0')
+                        self.debit = \
                             - self.party.account_payable.currency.round(amount)
-                    res['account'] = self.party.account_payable.id
-                    res['account.rec_name'] = \
-                        self.party.account_payable.rec_name
+                    self.account = self.party.account_payable
 
         if self.party and self.debit:
             if self.debit > Decimal('0.0'):
-                if 'account' not in res:
-                    res['account'] = self.party.account_receivable.id
-                    res['account.rec_name'] = \
-                        self.party.account_receivable.rec_name
+                if not self.account:
+                    self.account = self.party.account_receivable
             else:
-                if 'account' not in res:
-                    res['account'] = self.party.account_payable.id
-                    res['account.rec_name'] = \
-                        self.party.account_payable.rec_name
+                if not self.account:
+                    self.account = self.party.account_payable
 
         if self.party and self.credit:
             if self.credit > Decimal('0.0'):
-                if 'account' not in res:
-                    res['account'] = self.party.account_payable.id
-                    res['account.rec_name'] = \
-                        self.party.account_payable.rec_name
+                if not self.account:
+                    self.account = self.party.account_payable
             else:
-                if 'account' not in res:
-                    res['account'] = self.party.account_receivable.id
-                    res['account.rec_name'] = \
-                        self.party.account_receivable.rec_name
+                if not self.account:
+                    self.account = self.party.account_receivable
 
         journal = None
         if self.journal:
@@ -1105,16 +1110,11 @@ class Line(ModelSQL, ModelView):
             journal = Journal(Transaction().context.get('journal'))
         if journal and self.party:
             if journal.type == 'revenue':
-                if 'account' not in res:
-                    res['account'] = self.party.account_receivable.id
-                    res['account.rec_name'] = \
-                        self.party.account_receivable.rec_name
+                if not self.account:
+                    self.account = self.party.account_receivable
             elif journal.type == 'expense':
-                if 'account' not in res:
-                    res['account'] = self.party.account_payable.id
-                    res['account.rec_name'] = \
-                        self.party.account_payable.rec_name
-        return res
+                if not self.account:
+                    self.account = self.party.account_payable
 
     def get_move_field(self, name):
         field = getattr(self.__class__, name)
@@ -1165,6 +1165,23 @@ class Line(ModelSQL, ModelView):
     order_origin = _order_move_field('origin')
     order_move_state = _order_move_field('state')
 
+    def get_amount(self, name):
+        sign = 1 if self.account.type.display_balance == 'debit-credit' else -1
+        if self.amount_second_currency is not None:
+            return self.amount_second_currency * sign
+        else:
+            return self.debit - self.credit * sign
+
+    def get_amount_currency(self, name):
+        if self.second_currency:
+            currency = self.second_currency
+        else:
+            currency = self.account.currency
+        if name == 'amount_currency':
+            return currency.id
+        elif name == 'amount_currency_digits':
+            return currency.digits
+
     def get_rec_name(self, name):
         if self.debit > self.credit:
             return self.account.rec_name
@@ -1193,6 +1210,7 @@ class Line(ModelSQL, ModelView):
             fiscalyears = FiscalYear.search([
                     ('start_date', '<=', Transaction().context['date']),
                     ('end_date', '>=', Transaction().context['date']),
+                    ('company', '=', Transaction().context.get('company')),
                     ], limit=1)
 
             fiscalyear_id = fiscalyears and fiscalyears[0].id or 0
@@ -1239,6 +1257,7 @@ class Line(ModelSQL, ModelView):
             if not Transaction().context.get('fiscalyear'):
                 fiscalyears = FiscalYear.search([
                     ('state', '=', 'open'),
+                    ('company', '=', Transaction().context.get('company')),
                     ])
                 fiscalyear_ids = [f.id for f in fiscalyears] or [0]
             else:
@@ -1281,7 +1300,7 @@ class Line(ModelSQL, ModelView):
         if not self.account.active:
             self.raise_user_error('move_inactive_account', (
                     self.account.rec_name,))
-        if bool(self.party) != self.account.party_required:
+        if bool(self.party) != bool(self.account.party_required):
             error = 'party_set' if self.party else 'party_required'
             self.raise_user_error(error, self.rec_name)
 
@@ -1421,6 +1440,34 @@ class Line(ModelSQL, ModelView):
         journal_period, = journal_periods
         return value + ': ' + journal_period.rec_name
 
+    @classmethod
+    def view_toolbar_get(cls):
+        pool = Pool()
+        Template = pool.get('account.move.template')
+
+        toolbar = super(Line, cls).view_toolbar_get()
+
+        # Add a wizard entry for each templates
+        context = Transaction().context
+        company = context.get('company')
+        journal = context.get('journal')
+        period = context.get('period')
+        if company and journal and period:
+            templates = Template.search([
+                    ('company', '=', company),
+                    ('journal', '=', journal),
+                    ])
+            for template in templates:
+                action = toolbar['action']
+                # Use template id for action id to auto-select the template
+                action.append({
+                        'name': template.name,
+                        'type': 'ir.action.wizard',
+                        'wiz_name': 'account.move.template.create',
+                        'id': template.id,
+                        })
+        return toolbar
+
     @classmethod
     def fields_view_get(cls, view_id=None, view_type='form'):
         Journal = Pool().get('account.journal')
@@ -1460,6 +1507,11 @@ class Line(ModelSQL, ModelView):
             result['fields'] = cls.fields_get(fields_names=list(fields))
         return result
 
+    @classmethod
+    def view_attributes(cls):
+        return [('/tree[@on_write="on_write"]', 'colors',
+                If(Eval('state') == 'draft', 'red', 'black'))]
+
     @classmethod
     def reconcile(cls, lines, journal=None, date=None, account=None,
             description=None):
@@ -1474,7 +1526,7 @@ class Line(ModelSQL, ModelView):
                 cls.raise_user_error('already_reconciled',
                         error_args=(line.move.number, line.id,))
 
-        lines = lines[:]
+        lines = list(lines)
         reconcile_account = None
         reconcile_party = None
         amount = Decimal('0.0')
@@ -1856,7 +1908,7 @@ class Reconcile(Wizard):
     def _default_lines(self):
         'Return the larger list of lines which can be reconciled'
         currency = self.show.account.company.currency
-        chunk = config.getint('account', 'reconciliation_chunk', 10)
+        chunk = config.getint('account', 'reconciliation_chunk', default=10)
         # Combination is exponential so it must be limited to small number
         default = []
         for lines in grouped_slice(self._all_lines(), chunk):
@@ -1889,10 +1941,11 @@ class Reconcile(Wizard):
 class ReconcileShow(ModelView):
     'Reconcile'
     __name__ = 'account.reconcile.show'
-    accounts = fields.One2Many('account.account', None, 'Account',
+    accounts = fields.Many2Many('account.account', None, None, 'Account',
         readonly=True)
     account = fields.Many2One('account.account', 'Account', readonly=True)
-    parties = fields.One2Many('party.party', None, 'Parties', readonly=True)
+    parties = fields.Many2Many('party.party', None, None, 'Parties',
+        readonly=True)
     party = fields.Many2One('party.party', 'Party', readonly=True)
     lines = fields.Many2Many('account.move.line', None, None, 'Lines',
         domain=[
@@ -1937,9 +1990,20 @@ class ReconcileShow(ModelView):
 class CancelMoves(Wizard):
     'Cancel Moves'
     __name__ = 'account.move.cancel'
-    start_state = 'cancel'
+    start_state = 'default'
+    default = StateView('account.move.cancel.default',
+        'account.move_cancel_default_view_form', [
+            Button('Cancel', 'end', 'tryton-cancel'),
+            Button('OK', 'cancel', 'tryton-ok', default=True),
+            ])
     cancel = StateTransition()
 
+    def default_cancel(self, move):
+        default = {}
+        if self.default.description:
+            default['description'] = self.default.description
+        return default
+
     def transition_cancel(self):
         pool = Pool()
         Move = pool.get('account.move')
@@ -1947,7 +2011,8 @@ class CancelMoves(Wizard):
 
         moves = Move.browse(Transaction().context['active_ids'])
         for move in moves:
-            cancel_move = move.cancel()
+            default = self.default_cancel(move)
+            cancel_move = move.cancel(default=default)
             to_reconcile = defaultdict(list)
             for line in move.lines + cancel_move.lines:
                 if line.account.reconcile:
@@ -1957,6 +2022,12 @@ class CancelMoves(Wizard):
         return 'end'
 
 
+class CancelMovesDefault(ModelView):
+    'Cancel Moves'
+    __name__ = 'account.move.cancel.default'
+    description = fields.Char('Description')
+
+
 class FiscalYearLine(ModelSQL):
     'Fiscal Year - Move Line'
     __name__ = 'account.fiscalyear-account.move.line'
@@ -2038,15 +2109,16 @@ class GeneralJournal(Report):
                 order=[('date', 'ASC'), ('id', 'ASC')])
 
     @classmethod
-    def parse(cls, report, moves, data, localcontext):
+    def get_context(cls, records, data):
+        report_context = super(GeneralJournal, cls).get_context(records, data)
+
         Company = Pool().get('company.company')
 
         company = Company(data['company'])
 
-        localcontext['company'] = company
-        localcontext['digits'] = company.currency.digits
-        localcontext['from_date'] = data['from_date']
-        localcontext['to_date'] = data['to_date']
+        report_context['company'] = company
+        report_context['digits'] = company.currency.digits
+        report_context['from_date'] = data['from_date']
+        report_context['to_date'] = data['to_date']
 
-        return super(GeneralJournal, cls).parse(report, moves, data,
-            localcontext)
+        return report_context
