@@ -1,8 +1,12 @@
-#This file is part of Tryton.  The COPYRIGHT file at the top level of
-#this repository contains the full copyright notices and license terms.
+# This file is part of Tryton.  The COPYRIGHT file at the top level of
+# this repository contains the full copyright notices and license terms.
 import datetime
+from collections import namedtuple
 from decimal import Decimal
+
+from sql import Null
 from sql.aggregate import Sum
+from itertools import groupby
 
 from trytond.model import ModelView, ModelSQL, MatchMixin, fields
 from trytond.wizard import Wizard, StateView, StateAction, Button
@@ -231,7 +235,11 @@ class TaxCode(ModelSQL, ModelView):
 
     @classmethod
     def search_rec_name(cls, name, clause):
-        return ['OR',
+        if clause[1].startswith('!') or clause[1].startswith('not '):
+            bool_op = 'AND'
+        else:
+            bool_op = 'OR'
+        return [bool_op,
             ('code',) + tuple(clause[1:]),
             (cls._rec_name,) + tuple(clause[1:]),
             ]
@@ -366,6 +374,11 @@ class TaxTemplate(ModelSQL, ModelView):
         ('fixed', 'Fixed'),
         ('none', 'None'),
         ], 'Type', required=True)
+    update_unit_price = fields.Boolean('Update Unit Price',
+        states={
+            'invisible': Bool(Eval('parent')),
+            },
+        depends=['parent'])
     parent = fields.Many2One('account.tax.template', 'Parent')
     childs = fields.One2Many('account.tax.template', 'parent', 'Children')
     invoice_account = fields.Many2One('account.account.template',
@@ -394,6 +407,10 @@ class TaxTemplate(ModelSQL, ModelView):
         super(TaxTemplate, cls).__setup__()
         cls._order.insert(0, ('sequence', 'ASC'))
         cls._order.insert(0, ('account', 'ASC'))
+        cls._error_messages.update({
+                'update_unit_price_with_parent': ('"Update Unit Price" can '
+                    'not be set on tax "%(template)s" which has a parent.'),
+                })
 
     @classmethod
     def __register__(cls, module_name):
@@ -405,7 +422,7 @@ class TaxTemplate(ModelSQL, ModelView):
         # Migration from 1.0 group is no more required
         table.not_null_action('group', action='remove')
 
-        #Migration from 2.4: drop required on sequence
+        # Migration from 2.4: drop required on sequence
         table.not_null_action('sequence', action='remove')
 
         # Migration from 2.8: rename percentage into rate
@@ -416,10 +433,22 @@ class TaxTemplate(ModelSQL, ModelView):
                     values=[sql_table.percentage / 100]))
             table.drop_column('percentage')
 
+    @classmethod
+    def validate(cls, tax_templates):
+        super(TaxTemplate, cls).validate(tax_templates)
+        for tax_template in tax_templates:
+            tax_template.check_update_unit_price()
+
+    def check_update_unit_price(self):
+        if self.update_unit_price and self.parent:
+            self.raise_user_error('update_unit_price_with_parent', {
+                    'template': self.rec_name,
+                    })
+
     @staticmethod
     def order_sequence(tables):
         table, _ = tables[None]
-        return [table.sequence == None, table.sequence]
+        return [table.sequence == Null, table.sequence]
 
     @staticmethod
     def default_type():
@@ -445,6 +474,10 @@ class TaxTemplate(ModelSQL, ModelView):
     def default_credit_note_tax_sign():
         return Decimal('1')
 
+    @staticmethod
+    def default_update_unit_price():
+        return False
+
     def _get_tax_value(self, tax=None):
         '''
         Set values for tax creation.
@@ -453,7 +486,7 @@ class TaxTemplate(ModelSQL, ModelView):
         for field in ('name', 'description', 'sequence', 'amount',
                 'rate', 'type', 'invoice_base_sign', 'invoice_tax_sign',
                 'credit_note_base_sign', 'credit_note_tax_sign',
-                'start_date', 'end_date'):
+                'start_date', 'end_date', 'update_unit_price'):
             if not tax or getattr(tax, field) != getattr(self, field):
                 res[field] = getattr(self, field)
         for field in ('group',):
@@ -595,6 +628,13 @@ class Tax(ModelSQL, ModelView):
         ('fixed', 'Fixed'),
         ('none', 'None'),
         ], 'Type', required=True)
+    update_unit_price = fields.Boolean('Update Unit Price',
+        states={
+            'invisible': Bool(Eval('parent')),
+            },
+        depends=['parent'],
+        help=('If checked then the unit price for further tax computation will'
+            'be modified by this tax'))
     parent = fields.Many2One('account.tax', 'Parent', ondelete='CASCADE')
     childs = fields.One2Many('account.tax', 'parent', 'Children')
     company = fields.Many2One('company.company', 'Company', required=True,
@@ -625,9 +665,13 @@ class Tax(ModelSQL, ModelView):
         depends=['company', 'type'])
     invoice_base_code = fields.Many2One('account.tax.code',
         'Invoice Base Code',
+        domain=[
+            ('company', '=', Eval('company', -1)),
+            ],
         states={
             'readonly': Eval('type') == 'none',
-            }, depends=['type'])
+            },
+        depends=['type', 'company'])
     invoice_base_sign = fields.Numeric('Invoice Base Sign', digits=(2, 0),
         help='Usualy 1 or -1',
         states={
@@ -636,9 +680,13 @@ class Tax(ModelSQL, ModelView):
             }, depends=['type'])
     invoice_tax_code = fields.Many2One('account.tax.code',
         'Invoice Tax Code',
+        domain=[
+            ('company', '=', Eval('company', -1)),
+            ],
         states={
             'readonly': Eval('type') == 'none',
-            }, depends=['type'])
+            },
+        depends=['type', 'company'])
     invoice_tax_sign = fields.Numeric('Invoice Tax Sign', digits=(2, 0),
         help='Usualy 1 or -1',
         states={
@@ -647,9 +695,13 @@ class Tax(ModelSQL, ModelView):
             }, depends=['type'])
     credit_note_base_code = fields.Many2One('account.tax.code',
         'Credit Note Base Code',
+        domain=[
+            ('company', '=', Eval('company', -1)),
+            ],
         states={
             'readonly': Eval('type') == 'none',
-            }, depends=['type'])
+            },
+        depends=['type', 'company'])
     credit_note_base_sign = fields.Numeric('Credit Note Base Sign',
         digits=(2, 0), help='Usualy 1 or -1',
         states={
@@ -658,9 +710,13 @@ class Tax(ModelSQL, ModelView):
             }, depends=['type'])
     credit_note_tax_code = fields.Many2One('account.tax.code',
         'Credit Note Tax Code',
+        domain=[
+            ('company', '=', Eval('company', -1)),
+            ],
         states={
             'readonly': Eval('type') == 'none',
-            }, depends=['type'])
+            },
+        depends=['type', 'company'])
     credit_note_tax_sign = fields.Numeric('Credit Note Tax Sign',
         digits=(2, 0), help='Usualy 1 or -1',
         states={
@@ -673,6 +729,10 @@ class Tax(ModelSQL, ModelView):
     def __setup__(cls):
         super(Tax, cls).__setup__()
         cls._order.insert(0, ('sequence', 'ASC'))
+        cls._error_messages.update({
+                'update_unit_price_with_parent': ('"Update Unit Price" can '
+                    'not be set on tax "%(template)s" which has a parent.'),
+                })
 
     @classmethod
     def __register__(cls, module_name):
@@ -695,10 +755,22 @@ class Tax(ModelSQL, ModelView):
                     values=[sql_table.percentage / 100]))
             table.drop_column('percentage')
 
+    @classmethod
+    def validate(cls, taxes):
+        super(Tax, cls).validate(taxes)
+        for tax in taxes:
+            tax.check_update_unit_price()
+
+    def check_update_unit_price(self):
+        if self.parent and self.update_unit_price:
+            self.raise_user_error('update_unit_price_with_parent', {
+                    'tax': self.rec_name,
+                    })
+
     @staticmethod
     def order_sequence(tables):
         table, _ = tables[None]
-        return [table.sequence == None, table.sequence]
+        return [table.sequence == Null, table.sequence]
 
     @staticmethod
     def default_active():
@@ -728,6 +800,10 @@ class Tax(ModelSQL, ModelView):
     def default_credit_note_tax_sign():
         return Decimal('1')
 
+    @staticmethod
+    def default_update_unit_price():
+        return False
+
     @staticmethod
     def default_company():
         return Transaction().context.get('company')
@@ -754,26 +830,81 @@ class Tax(ModelSQL, ModelView):
                 'tax': self,
                 }
 
+    def _group_taxes(self):
+        'Key method used to group taxes'
+        return (self.sequence,)
+
     @classmethod
     def _unit_compute(cls, taxes, price_unit, date):
         res = []
+        for _, group_taxes in groupby(taxes, key=cls._group_taxes):
+            unit_price_variation = 0
+            for tax in group_taxes:
+                start_date = tax.start_date or datetime.date.min
+                end_date = tax.end_date or datetime.date.max
+                if not (start_date <= date <= end_date):
+                    continue
+                if tax.type != 'none':
+                    value = tax._process_tax(price_unit)
+                    res.append(value)
+                    if tax.update_unit_price:
+                        unit_price_variation += value['amount']
+                if len(tax.childs):
+                    res.extend(cls._unit_compute(tax.childs, price_unit, date))
+            price_unit += unit_price_variation
+        return res
+
+    @classmethod
+    def _reverse_rate_amount(cls, taxes, date):
+        rate, amount = 0, 0
         for tax in taxes:
             start_date = tax.start_date or datetime.date.min
             end_date = tax.end_date or datetime.date.max
             if not (start_date <= date <= end_date):
                 continue
-            if tax.type != 'none':
-                res.append(tax._process_tax(price_unit))
-            if len(tax.childs):
-                res.extend(cls._unit_compute(tax.childs, price_unit, date))
-        return res
+
+            if tax.type == 'percentage':
+                rate += tax.rate
+            elif tax.type == 'fixed':
+                amount += tax.amount
+
+            if tax.childs:
+                child_rate, child_amount = cls._reverse_rate_amount(
+                    tax.childs, date)
+                rate += child_rate
+                amount += child_amount
+        return rate, amount
 
     @classmethod
-    def sort_taxes(cls, taxes):
+    def _reverse_unit_compute(cls, price_unit, taxes, date):
+        rate, amount = 0, 0
+        update_unit_price = False
+        unit_price_variation_amount = 0
+        unit_price_variation_rate = 0
+        for _, group_taxes in groupby(taxes, key=cls._group_taxes):
+            group_taxes = list(group_taxes)
+            g_rate, g_amount = cls._reverse_rate_amount(group_taxes, date)
+            if update_unit_price:
+                g_amount += unit_price_variation_amount * g_rate
+                g_rate += unit_price_variation_rate * g_rate
+
+            g_update_unit_price = any(t.update_unit_price for t in group_taxes)
+            update_unit_price |= g_update_unit_price
+            if g_update_unit_price:
+                unit_price_variation_amount += g_amount
+                unit_price_variation_rate += g_rate
+
+            rate += g_rate
+            amount += g_amount
+
+        return (price_unit - amount) / (1 + rate)
+
+    @classmethod
+    def sort_taxes(cls, taxes, reverse=False):
         '''
         Return a list of taxes sorted
         '''
-        return sorted(taxes, key=lambda t: (t.sequence, t.id))
+        return sorted(taxes, key=lambda t: (t.sequence, t.id), reverse=reverse)
 
     @classmethod
     def compute(cls, taxes, price_unit, quantity, date=None):
@@ -796,6 +927,18 @@ class Tax(ModelSQL, ModelView):
             row['amount'] *= quantity
         return res
 
+    @classmethod
+    def reverse_compute(cls, price_unit, taxes, date=None):
+        '''
+        Reverse compute the price_unit for taxes at the date.
+        '''
+        pool = Pool()
+        Date = pool.get('ir.date')
+        if date is None:
+            date = Date.today()
+        taxes = cls.sort_taxes(taxes)
+        return cls._reverse_unit_compute(price_unit, taxes, date)
+
     def update_tax(self, template2tax_code, template2account,
             template2tax=None):
         '''
@@ -908,6 +1051,113 @@ class Tax(ModelSQL, ModelView):
                 template2tax=template2tax)
 
 
+class _TaxKey(dict):
+
+    def __init__(self, **kwargs):
+        self.update(kwargs)
+
+    def _key(self):
+        return (self['base_code'], self['base_sign'],
+            self['tax_code'], self['tax_sign'],
+            self['account'], self['tax'])
+
+    def __eq__(self, other):
+        if isinstance(other, _TaxKey):
+            return self._key() == other._key()
+        return self._key() == other
+
+    def __hash__(self):
+        return hash(self._key())
+
+_TaxableLine = namedtuple('_TaxableLine', ('taxes', 'unit_price', 'quantity'))
+
+
+class TaxableMixin(object):
+
+    @property
+    def taxable_lines(self):
+        """A list of tuples where
+            - the first element is the taxes applicable
+            - the second element is the line unit price
+            - the third element is the line quantity
+        """
+        return []
+
+    @property
+    def tax_type(self):
+        "The taxation type it can be 'invoice' or 'credit_note'"
+        return None
+
+    @property
+    def currency(self):
+        "The currency used by the taxable object"
+        return None
+
+    @property
+    def tax_date(self):
+        "Date to use when computing the tax"
+        pool = Pool()
+        Date = pool.get('ir.date')
+        return Date.today()
+
+    def _get_tax_context(self):
+        return {}
+
+    @staticmethod
+    def _compute_tax_line(type_, amount, base, tax):
+        assert type_ in ('invoice', 'credit_note')
+
+        line = {}
+        line['manual'] = False
+        line['description'] = tax.description
+        line['base'] = base
+        line['amount'] = amount
+        line['tax'] = tax.id if tax else None
+
+        for attribute in ['base_code', 'tax_code', 'account']:
+            value = getattr(tax, '%s_%s' % (type_, attribute), None)
+            line[attribute] = value.id if value else None
+
+        for attribute in ['base_sign', 'tax_sign']:
+            value = getattr(tax, '%s_%s' % (type_, attribute), None)
+            line[attribute] = value
+
+        return _TaxKey(**line)
+
+    def _round_taxes(self, taxes):
+        if not self.currency:
+            return
+        for taxline in taxes.itervalues():
+            for attribute in ('base', 'amount'):
+                taxline[attribute] = self.currency.round(taxline[attribute])
+
+    def _get_taxes(self):
+        pool = Pool()
+        Tax = pool.get('account.tax')
+        Configuration = pool.get('account.configuration')
+
+        config = Configuration(1)
+        taxes = {}
+        with Transaction().set_context(self._get_tax_context()):
+            taxable_lines = [_TaxableLine(*params)
+                for params in self.taxable_lines]
+            for line in taxable_lines:
+                l_taxes = Tax.compute(line.taxes, line.unit_price,
+                    line.quantity, self.tax_date)
+                for tax in l_taxes:
+                    taxline = self._compute_tax_line(self.tax_type, **tax)
+                    if taxline not in taxes:
+                        taxes[taxline] = taxline
+                    else:
+                        taxes[taxline]['base'] += taxline['base']
+                        taxes[taxline]['amount'] += taxline['amount']
+                if config.tax_rounding == 'line':
+                    self._round_taxes(taxes)
+        if config.tax_rounding == 'document':
+            self._round_taxes(taxes)
+        return taxes
+
+
 class TaxLine(ModelSQL, ModelView):
     'Tax Line'
     __name__ = 'account.tax.line'
@@ -917,11 +1167,21 @@ class TaxLine(ModelSQL, ModelView):
     amount = fields.Numeric('Amount', digits=(16, Eval('currency_digits', 2)),
         required=True, depends=['currency_digits'])
     code = fields.Many2One('account.tax.code', 'Code', select=True,
-        required=True)
+        required=True,
+        domain=[
+            ('company', '=', Eval('company', -1)),
+            ],
+        depends=['company'])
     tax = fields.Many2One('account.tax', 'Tax', select=True,
-        ondelete='RESTRICT')
+        ondelete='RESTRICT',
+        domain=[
+            ('company', '=', Eval('company', -1)),
+            ],
+        depends=['company'])
     move_line = fields.Many2One('account.move.line', 'Move Line',
             required=True, select=True, ondelete='CASCADE')
+    company = fields.Function(fields.Many2One('company.company', 'Company'),
+        'on_change_with_company')
 
     @fields.depends('move_line')
     def on_change_with_currency_digits(self, name=None):
@@ -931,9 +1191,12 @@ class TaxLine(ModelSQL, ModelView):
 
     @fields.depends('tax')
     def on_change_tax(self):
-        return {
-            'code': None,
-            }
+        self.code = None
+
+    @fields.depends('_parent_move_line.account')
+    def on_change_with_company(self, name=None):
+        if self.move_line:
+            return self.move_line.account.company.id
 
 
 class TaxRuleTemplate(ModelSQL, ModelView):
@@ -1142,7 +1405,7 @@ class TaxRuleLineTemplate(ModelSQL, ModelView):
     @staticmethod
     def order_sequence(tables):
         table, _ = tables[None]
-        return [table.sequence == None, table.sequence]
+        return [table.sequence == Null, table.sequence]
 
     def _get_tax_rule_line_value(self, rule_line=None):
         '''
@@ -1256,7 +1519,7 @@ class TaxRuleLine(ModelSQL, ModelView, MatchMixin):
     @staticmethod
     def order_sequence(tables):
         table, _ = tables[None]
-        return [table.sequence == None, table.sequence]
+        return [table.sequence == Null, table.sequence]
 
     def match(self, pattern):
         if 'group' in pattern and not self.group:
